#
# BeScope TCL Server
#
# server.tcl
#
# nCk Research, LLC 2014(C)
#
# Written By: Ryan Belanger: ryan.belanger@nck-research.com
#             Matt Chabot: matt.chabot@nck-research.com
#             06.26.2014
#
# The BeScope TCL Server is design to be run with Altera 13.1
# system-console.exe executable
#
# Usage: ..\..\system-console.exe --script server.tcl
#
# This file in conjunction with Altera's system-console.exe executable will
# launch a TCP server listening on port 2540, that will accept two ASCII
# messages "SET" and "GET".
#
# The "SET" command will be in format "SET0xXXXXXXXX\n", where the 0xXX...
# denotes a 32-bit value to written into the FPGA controlling the function
# of the BeScope hardware through the FPGA on the BeMicro hardware platform
#
# The "GET" command will be in format "GET\n", where the command will
# generate a system-console master_read_memory byte-wise command of a 32bit
# 1K memory( 4096 byte results ) allocated in the BeMicro memory. The byte
# results are returned in ASCII string formatted, "0x00 0x01 0x02..."
#

# Procedure: Server - Facilitates socket server launching
#
proc Server {port} {
	global echo 
    set echo(main) [socket -server Accept $port]
    vwait forever
}

# Procedure: Accept - Facilitates accepting socket connection
#                     with the BeScope Application python software
#
proc Accept {sock addr port} {

    global echo

    # Record the client's information

    puts "Accept $sock from $addr port $port"
    set echo(addr,$sock) [list $addr $port]

    # Ensure that each "puts" by the server
    # results in a network transmission

    fconfigure $sock -buffering line -buffersize 2048

    # Set up a callback for when the client sends data

    fileevent $sock readable [list IncomingPacket $sock]
}

# Procdure: IncomingPacket - Facilitates socket communication with the
#                            BeScope Application python software, the
#                            "SET" and "GET" command are parsed and the
#                            system-console.exe commands are generated
#                            within the procedure.
#
proc IncomingPacket {sock} {

	global i
    global echo
	global nm
	global d

	# check for socket close call from BeScope Application software
	#
    if {[eof $sock] || [catch {gets $sock line}]} {

		# close TCL side TCP socket server connection
		#
		close $sock
		puts "Close $echo(addr,$sock)"

		#unset echo(addr,$sock)
		#close_service master $nm

	# recieved packet must not be a socket close communication,
	# check for "SET" or "GET" compliant commands
	#
    } else {
		# process "GET" command
		#
		if {[string compare [string range $line 0 2] "GET"]==0} {
			#puts "GET rx"
			#puts $line
			set a [master_read_memory $nm 0x10000 4096]
			set b [master_read_memory $nm 0x20000 4096]
			set ab "${a} ${b}"
			puts $sock $ab
		}
		# process "SET" command
		#
		if {[string compare [string range $line 0 2] "SET"]==0} {
			#puts "SET found"
			set r [string range $line 5 13]
			#puts $r
			scan $r "%8x" val
			master_write_32 $nm 0x00000 $val
		}
    }
}

puts "BeScope TCL-TCP Server v0.0.1"
puts "nCk Research, LLC. 2014(C)\n"

set nm [lindex [get_service_paths master] 0]
set msg "Found: ${nm}"
puts $msg

open_service master $nm

puts "Launching TCL-TCP Server on port 2540"

# launch the TCP server under TCL, awaiting for connection from
# the BeScope Application software, wait forever, never closing
# close_service master $nm, once connected to the BeMicro and
# BeScope hardware assembly.
#
Server 2540
